import axios from 'axios';
import SyncClient from '../clients/sync.client.js';
import proxyService from '../services/proxy.service.js';
import integrationPostConnectionScript from '../integrations/scripts/connection/connection.manager.js';
import webhookService from '../services/notification/webhook.service.js';
import { SpanTypes } from '../utils/telemetry.js';
import { isCloud, isLocal, isEnterprise } from '@nangohq/utils/dist/environment/detection.js';
import { resultOk, resultErr } from '../utils/result.js';
import { NangoError } from '../utils/error.js';
export const connectionCreated = async (connection, provider, activityLogId, options = { initiateSync: true, runPostConnectionScript: true }) => {
    const hosted = !isCloud && !isLocal && !isEnterprise;
    if (options.initiateSync === true && !hosted) {
        const syncClient = await SyncClient.getInstance();
        syncClient?.initiate(connection.id);
    }
    if (options.runPostConnectionScript === true) {
        integrationPostConnectionScript(connection, provider);
    }
    await webhookService.sendAuthUpdate(connection, provider, true, activityLogId);
};
export const connectionCreationFailed = async (connection, provider, activityLogId) => {
    await webhookService.sendAuthUpdate(connection, provider, false, activityLogId);
};
export const connectionTest = async (provider, template, credentials, connectionId, providerConfigKey, environment_id, connection_config, tracer) => {
    const providerVerification = template?.proxy?.verification;
    if (!providerVerification) {
        return resultOk(true);
    }
    const active = tracer.scope().active();
    const span = tracer.startSpan(SpanTypes.CONNECTION_TEST, {
        childOf: active,
        tags: {
            'nango.provider': provider,
            'nango.providerConfigKey': providerConfigKey,
            'nango.connectionId': connectionId
        }
    });
    const { method, endpoint } = providerVerification;
    const connection = {
        id: -1,
        provider_config_key: providerConfigKey,
        connection_id: connectionId,
        credentials,
        connection_config,
        environment_id
    };
    const configBody = {
        endpoint,
        method: method?.toUpperCase(),
        template,
        token: credentials,
        provider: provider,
        providerConfigKey,
        connectionId,
        headers: {
            'Content-Type': 'application/json'
        },
        connection
    };
    const internalConfig = {
        provider,
        connection
    };
    try {
        const { response } = await proxyService.route(configBody, internalConfig);
        if (axios.isAxiosError(response)) {
            span.setTag('nango.error', response);
            const error = new NangoError('connection_test_failed', response, response.response?.status);
            return resultErr(error);
        }
        if (!response) {
            const error = new NangoError('connection_test_failed');
            span.setTag('nango.error', response);
            return resultErr(error);
        }
        if (response.status && (response?.status < 200 || response?.status > 300)) {
            const error = new NangoError('connection_test_failed');
            span.setTag('nango.error', response);
            return resultErr(error);
        }
        return resultOk(true);
    }
    catch (e) {
        const error = new NangoError('connection_test_failed');
        span.setTag('nango.error', e);
        return resultErr(error);
    }
    finally {
        span.finish();
    }
};
//# sourceMappingURL=hooks.js.map