import { schema } from '../../../db/database.js';
import { getRecordsByExternalIds, verifyUniqueKeysAreUnique } from './records.service.js';
import { createActivityLogMessage } from '../../activity/activity.service.js';
import { markRecordsForDeletion, syncCreatedAtForAddedRecords, syncUpdateAtForChangedRecords } from './delete.service.js';
import encryptionManager from '../../../utils/encryption.manager.js';
import { logger } from '../../../index.js';
/**
 * Upsert
 */
export async function upsert(response, dbTable, uniqueKey, nangoConnectionId, model, activityLogId, environment_id, track_deletes = false, softDelete = false) {
    const responseWithoutDuplicates = await removeDuplicateKey(response, uniqueKey, activityLogId, environment_id, model);
    if (!responseWithoutDuplicates || responseWithoutDuplicates.length === 0) {
        return {
            success: false,
            error: `There are no records to upsert because there were no records that were not duplicates to insert, but there were ${response.length} records received for the "${model}" model.`
        };
    }
    const comparisonTable = track_deletes ? '_nango_sync_data_records_deletes' : dbTable;
    const addedKeys = await getAddedKeys(responseWithoutDuplicates, comparisonTable, uniqueKey, nangoConnectionId, model);
    const updatedKeys = await getUpdatedKeys(responseWithoutDuplicates, comparisonTable, uniqueKey, nangoConnectionId, model);
    try {
        if (track_deletes) {
            await markRecordsForDeletion(nangoConnectionId, model);
        }
        const encryptedRecords = encryptionManager.encryptDataRecords(responseWithoutDuplicates);
        const results = await schema()
            .from(dbTable)
            .insert(encryptedRecords, ['id', 'external_id'])
            .onConflict(['nango_connection_id', 'external_id', 'model'])
            .merge()
            .returning(['id', 'external_id']);
        const affectedInternalIds = results.map((tuple) => tuple.id);
        const affectedExternalIds = results.map((tuple) => tuple.external_id);
        if (softDelete) {
            return {
                success: true,
                summary: {
                    deletedKeys: [...addedKeys, ...updatedKeys],
                    addedKeys: [],
                    updatedKeys: [],
                    affectedInternalIds,
                    affectedExternalIds
                }
            };
        }
        if (track_deletes) {
            // we need to main the created at date of the existing records so we know what
            // was added and what was updated
            await syncUpdateAtForChangedRecords(nangoConnectionId, model, uniqueKey, updatedKeys);
            await syncCreatedAtForAddedRecords(nangoConnectionId, model, uniqueKey, addedKeys);
        }
        return {
            success: true,
            summary: {
                addedKeys,
                updatedKeys,
                deletedKeys: [],
                affectedInternalIds,
                affectedExternalIds
            }
        };
    }
    catch (error) {
        let errorMessage = `Failed to upsert records to table ${dbTable}.\n`;
        errorMessage += `Model: ${model}, Unique Key: ${uniqueKey}, Nango Connection ID: ${nangoConnectionId}.\n`;
        errorMessage += `Attempted to insert/update/delete: ${responseWithoutDuplicates.length} records\n`;
        if (error.code)
            errorMessage += `Error code: ${error.code}.\n`;
        logger.error(`${errorMessage}${error}`);
        let errorDetail = '';
        switch (error.code) {
            case '22001': {
                errorDetail = "String length exceeds the column's maximum length (string_data_right_truncation)";
                break;
            }
        }
        if (errorDetail)
            errorMessage += `Info: ${errorDetail}.\n`;
        return {
            success: false,
            error: errorMessage
        };
    }
}
export async function updateRecord(records, dbTable, uniqueKey, nangoConnectionId, model, activityLogId, environment_id) {
    const responseWithoutDuplicates = await removeDuplicateKey(records, uniqueKey, activityLogId, environment_id, model);
    if (!responseWithoutDuplicates || responseWithoutDuplicates.length === 0) {
        return {
            success: false,
            error: `There are no records to upsert because there were no records that were not duplicates to insert, but there were ${records.length} records received for the "${model}" model.`
        };
    }
    const updatedKeys = await getUpdatedKeys(responseWithoutDuplicates, dbTable, uniqueKey, nangoConnectionId, model);
    try {
        const recordsToUpdate = [];
        const rawOldRecords = await getRecordsByExternalIds(updatedKeys, nangoConnectionId, model);
        for (const rawOldRecord of rawOldRecords) {
            if (!rawOldRecord) {
                continue;
            }
            const { record: oldRecord } = rawOldRecord;
            const record = records.find((record) => record.external_id.toString() === oldRecord?.id?.toString());
            const newRecord = {
                ...rawOldRecord,
                json: {
                    ...oldRecord,
                    ...record?.json
                },
                updated_at: new Date()
            };
            delete newRecord.record;
            recordsToUpdate.push(newRecord);
        }
        const encryptedRecords = encryptionManager.encryptDataRecords(recordsToUpdate);
        const results = await schema()
            .from(dbTable)
            .insert(encryptedRecords, ['id', 'external_id'])
            .onConflict(['nango_connection_id', 'external_id', 'model'])
            .merge()
            .returning(['id', 'external_id']);
        const affectedInternalIds = results.map((tuple) => tuple.id);
        const affectedExternalIds = results.map((tuple) => tuple.external_id);
        return {
            success: true,
            summary: {
                addedKeys: [],
                updatedKeys,
                deletedKeys: [],
                affectedInternalIds,
                affectedExternalIds
            }
        };
    }
    catch (error) {
        let errorMessage = `Failed to update records to table ${dbTable}.\n`;
        errorMessage += `Model: ${model}, Unique Key: ${uniqueKey}, Nango Connection ID: ${nangoConnectionId}.\n`;
        errorMessage += `Attempted to update: ${responseWithoutDuplicates.length} records\n`;
        if ('code' in error)
            errorMessage += `Error code: ${error.code}.\n`;
        if ('detail' in error)
            errorMessage += `Detail: ${error.detail}.\n`;
        errorMessage += `Error Message: ${error.message}`;
        return {
            success: false,
            error: errorMessage
        };
    }
}
export async function removeDuplicateKey(response, uniqueKey, activityLogId, environment_id, model) {
    const { nonUniqueKeys } = verifyUniqueKeysAreUnique(response, uniqueKey);
    for (const nonUniqueKey of nonUniqueKeys) {
        await createActivityLogMessage({
            level: 'error',
            environment_id,
            activity_log_id: activityLogId,
            content: `There was a duplicate key found: ${nonUniqueKey}. This record will be ignore in relation to the model ${model}.`,
            timestamp: Date.now()
        });
    }
    const seen = new Set();
    const uniqueResponse = response.filter((item) => {
        const key = item[uniqueKey];
        return seen.has(key) ? false : seen.add(key);
    });
    return uniqueResponse;
}
/**
 * Compute Added Keys
 * @desc for any incoming payload use the provided unique to check against the rows
 * in the database and return the keys that are not in the database
 *
 */
export async function getAddedKeys(response, dbTable, uniqueKey, nangoConnectionId, model) {
    const keys = response.map((data) => String(data[uniqueKey]));
    const knownKeys = (await schema()
        .from(dbTable)
        .where('nango_connection_id', nangoConnectionId)
        .where('model', model)
        .whereIn('external_id', keys)
        .pluck('external_id'));
    const unknownKeys = keys?.filter((data) => !knownKeys.includes(data));
    return unknownKeys;
}
/**
 * Get Updated Keys
 * @desc generate an array of the keys that exist in the database and also in
 * the incoming payload that will be used to update the database.
 * Compare using the data_hash key
 *
 */
export async function getUpdatedKeys(response, dbTable, uniqueKey, nangoConnectionId, model) {
    const keys = response.map((data) => String(data[uniqueKey]));
    const keysWithHash = response.map((data) => [String(data[uniqueKey]), data['data_hash']]);
    const rowsToUpdate = await schema()
        .from(dbTable)
        .pluck('external_id')
        .where('nango_connection_id', nangoConnectionId)
        .where('model', model)
        .whereIn('external_id', keys)
        .whereNotIn(['external_id', 'data_hash'], keysWithHash);
    return rowsToUpdate;
}
//# sourceMappingURL=data.service.js.map