import * as uuid from 'uuid';
import sentry from '@sentry/node';
import { readFileSync } from 'fs';
import path from 'path';
import logger from '../logger/console.js';
import { NangoError } from './error.js';
import { isCloud, getEnvironmentId, getAccountIdAndEnvironmentIdFromSession, dirname, isApiAuthenticated, isUserAuthenticated } from './utils.js';
import environmentService from '../services/environment.service.js';
import accountService from '../services/account.service.js';
import userService from '../services/user.service.js';
export var ErrorSourceEnum;
(function (ErrorSourceEnum) {
    ErrorSourceEnum["PLATFORM"] = "platform";
    ErrorSourceEnum["CUSTOMER"] = "customer";
})(ErrorSourceEnum || (ErrorSourceEnum = {}));
class ErrorManager {
    constructor() {
        try {
            if (isCloud() && process.env['SENTRY_DNS']) {
                const packageVersion = JSON.parse(readFileSync(path.resolve(dirname(), '../../../package.json'), 'utf8')).version;
                sentry.init({
                    dsn: process.env['SENTRY_DNS'],
                    beforeSend(event, _) {
                        return event.user?.id === 'account-78' ? null : event;
                    },
                    environment: process.env['NODE_ENV'],
                    release: 'nango@' + packageVersion
                });
            }
        }
        catch (_) {
            return;
        }
    }
    report(e, config = { source: ErrorSourceEnum.PLATFORM }, tracer) {
        sentry.withScope(async function (scope) {
            if (config.environmentId || config.accountId) {
                let accountId;
                if (config.environmentId) {
                    const environmentName = await environmentService.getEnvironmentName(config.environmentId);
                    accountId = (await environmentService.getAccountIdFromEnvironment(config.environmentId));
                    sentry.setTag('environmentName', environmentName);
                }
                if (config.accountId && !config.environmentId) {
                    accountId = config.accountId;
                }
                const account = await accountService.getAccountById(accountId);
                if (!config.userId) {
                    const users = await userService.getUsersByAccountId(accountId);
                    sentry.setUser({
                        id: `account-${accountId}`,
                        organization: account?.name || '',
                        emails: users.map((user) => user.email).join(','),
                        userIds: users.map((user) => user.id).join(',')
                    });
                }
            }
            if (config.userId) {
                const user = await userService.getUserById(config.userId);
                sentry.setUser({
                    id: `user-${config.userId}`,
                    email: user?.email || '',
                    userId: user?.id
                });
            }
            sentry.setTag('source', config.source);
            if (config.operation) {
                sentry.setTag('operation', config.operation);
            }
            if (config.metadata) {
                const metadata = Object.entries(config.metadata).reduce((acc, [key, value]) => {
                    if (typeof value === 'object') {
                        acc[key] = JSON.stringify(value);
                    }
                    else {
                        acc[key] = value;
                    }
                    return acc;
                }, {});
                scope.setContext('metadata', metadata);
            }
            if (typeof e === 'string') {
                sentry.captureException(new Error(e));
            }
            else {
                sentry.captureException(e);
            }
        });
        logger.error(`Exception caught: ${JSON.stringify(e, Object.getOwnPropertyNames(e))}`);
        if (e instanceof Error && tracer) {
            // Log to datadog manually
            // https://github.com/DataDog/dd-trace-js/issues/1944
            const span = tracer.scope().active();
            if (span) {
                span.setTag('error', e);
            }
        }
    }
    errResFromNangoErr(res, err) {
        if (err) {
            logger.error(`Response error: ${JSON.stringify({ statusCode: err.status, type: err.type, payload: err.payload, message: err.message })}`);
            if (!err.message) {
                res.status(err.status || 500).send({ type: err.type, payload: err.payload });
            }
            else {
                res.status(err.status || 500).send({ error: err.message, type: err.type, payload: err.payload });
            }
        }
    }
    errRes(res, type) {
        const err = new NangoError(type);
        this.errResFromNangoErr(res, err);
    }
    async handleGenericError(err, req, res, tracer) {
        const errorId = uuid.v4();
        if (!(err instanceof Error)) {
            err = new NangoError('generic_error_malformed', errorId);
        }
        else if (!(err instanceof NangoError)) {
            err = new NangoError(err.message, errorId);
        }
        const nangoErr = err;
        if (isApiAuthenticated(res)) {
            const environmentId = getEnvironmentId(res);
            this.report(nangoErr, { source: ErrorSourceEnum.PLATFORM, environmentId, metadata: err.payload }, tracer);
        }
        else if (isUserAuthenticated(req)) {
            const { response, success, error } = await getAccountIdAndEnvironmentIdFromSession(req);
            if (!success || response === null) {
                this.report(error, { source: ErrorSourceEnum.PLATFORM, metadata: err.payload }, tracer);
            }
            else {
                const { environmentId } = response;
                this.report(nangoErr, { source: ErrorSourceEnum.PLATFORM, environmentId, metadata: err.payload }, tracer);
            }
        }
        else {
            this.report(nangoErr, { source: ErrorSourceEnum.PLATFORM, metadata: err.payload }, tracer);
        }
        const supportError = new NangoError('generic_error_support', errorId);
        this.errResFromNangoErr(res, supportError);
    }
    getExpressRequestContext(req) {
        const metadata = {};
        metadata['baseUrl'] = req.baseUrl;
        metadata['originalUrl'] = req.originalUrl;
        metadata['subdomains'] = req.subdomains;
        metadata['body'] = req.body;
        metadata['hostname'] = req.hostname;
        metadata['method'] = req.method;
        metadata['params'] = req.params;
        metadata['query'] = req.query;
        return metadata;
    }
}
export default new ErrorManager();
//# sourceMappingURL=error.manager.js.map