import path, { resolve } from 'path';
import { readFileSync } from 'fs';
import { fileURLToPath } from 'url';
import { NangoError } from './error.js';
import environmentService from '../services/environment.service.js';
import userService from '../services/user.service.js';
const PORT = process.env['SERVER_PORT'] || 3003;
export const localhostUrl = `http://localhost:${PORT}`;
export const cloudHost = 'https://api.nango.dev';
export const stagingHost = 'https://api-staging.nango.dev';
const accountIdLocalsKey = 'nangoAccountId';
const environmentIdLocalsKey = 'nangoEnvironmentId';
export var UserType;
(function (UserType) {
    UserType["Local"] = "localhost";
    UserType["SelfHosted"] = "self-hosted";
    UserType["Cloud"] = "cloud";
})(UserType || (UserType = {}));
export var NodeEnv;
(function (NodeEnv) {
    NodeEnv["Dev"] = "development";
    NodeEnv["Staging"] = "staging";
    NodeEnv["Prod"] = "production";
})(NodeEnv || (NodeEnv = {}));
export const AUTH_ENABLED = isCloud() || isEnterprise();
export const MANAGED_AUTH_ENABLED = isCloud() || isLocal();
export const JAVASCRIPT_AND_TYPESCRIPT_TYPES = {
    primitives: ['string', 'number', 'boolean', 'bigint', 'symbol', 'undefined', 'null'],
    aliases: ['String', 'Number', 'Boolean', 'BigInt', 'Symbol', 'Undefined', 'Null', 'bool', 'char', 'integer', 'int', 'date', 'object'],
    builtInObjects: ['Object', 'Array', 'Function', 'Date', 'RegExp', 'Map', 'Set', 'WeakMap', 'WeakSet', 'Promise', 'Symbol', 'Error'],
    utilityTypes: ['Record', 'Partial', 'Readonly', 'Pick']
};
export function isJsOrTsType(type) {
    if (!type) {
        return false;
    }
    const baseType = type.replace(/\[\]$/, '');
    const simpleTypes = Object.values(JAVASCRIPT_AND_TYPESCRIPT_TYPES).flat();
    if (simpleTypes.includes(baseType)) {
        return true;
    }
    const typesWithGenerics = [...JAVASCRIPT_AND_TYPESCRIPT_TYPES.builtInObjects, ...JAVASCRIPT_AND_TYPESCRIPT_TYPES.utilityTypes];
    const genericTypeRegex = new RegExp(`^(${typesWithGenerics.join('|')})<.+>$`);
    return genericTypeRegex.test(baseType);
}
export function getEnv() {
    if (isStaging()) {
        return NodeEnv.Staging;
    }
    else if (isProd()) {
        return NodeEnv.Prod;
    }
    else {
        return NodeEnv.Dev;
    }
}
export function isLocal() {
    return getBaseUrl() === localhostUrl;
}
export function isCloud() {
    return process.env['NANGO_CLOUD']?.toLowerCase() === 'true';
}
export function isEnterprise() {
    return process.env['NANGO_ENTERPRISE']?.toLowerCase() === 'true';
}
export function integrationFilesAreRemote() {
    const useS3 = Boolean(process.env['AWS_REGION'] && process.env['AWS_BUCKET_NAME']);
    return isEnterprise() && useS3;
}
export function isStaging() {
    return process.env['NODE_ENV'] === NodeEnv.Staging;
}
export function isHosted() {
    return !isCloud() && !isLocal() && !isEnterprise();
}
export function getPort() {
    if (process.env['SERVER_PORT']) {
        return +process.env['SERVER_PORT'];
    }
    else if (process.env['PORT']) {
        return +process.env['PORT']; // For Heroku (dynamic port)
    }
    else if (process.env['NANGO_PORT']) {
        return +process.env['NANGO_PORT']; // more friendly cli port name
    }
    else {
        return 3003;
    }
}
export function getServerPort() {
    if (process.env['SERVER_PORT']) {
        return +process.env['SERVER_PORT'];
    }
    else if (process.env['PORT']) {
        return +process.env['PORT']; // For Heroku (dynamic port)
    }
    else if (process.env['NANGO_PORT']) {
        return +process.env['NANGO_PORT']; // more friendly cli port name
    }
    else {
        return 3003;
    }
}
export function getPersistAPIUrl() {
    return process.env['PERSIST_SERVICE_URL'] || 'http://localhost:3007';
}
export function isDev() {
    return process.env['NODE_ENV'] === NodeEnv.Dev;
}
export function isProd() {
    return process.env['NODE_ENV'] === NodeEnv.Prod;
}
export function isTest() {
    return Boolean(process.env['CI'] !== undefined || process.env['VITEST']);
}
export function isBasicAuthEnabled() {
    return !isCloud() && process.env['NANGO_DASHBOARD_USERNAME'] && process.env['NANGO_DASHBOARD_PASSWORD'];
}
function getServerHost() {
    return process.env['SERVER_HOST'] || process.env['SERVER_RUN_MODE'] === 'DOCKERIZED' ? 'http://nango-server' : 'http://localhost';
}
export function getServerBaseUrl() {
    return getServerHost() + `:${getServerPort()}`;
}
export function getRedisUrl() {
    return process.env['NANGO_REDIS_URL'] || undefined;
}
export function isValidHttpUrl(str) {
    try {
        new URL(str);
        return true;
    }
    catch {
        return false;
    }
}
export function dirname() {
    return path.dirname(fileURLToPath(import.meta.url));
}
export function parseTokenExpirationDate(expirationDate) {
    if (expirationDate instanceof Date) {
        return expirationDate;
    }
    // UNIX timestamp
    if (typeof expirationDate === 'number') {
        return new Date(expirationDate * 1000);
    }
    // ISO 8601 string
    return new Date(expirationDate);
}
export function isTokenExpired(expireDate, bufferInSeconds) {
    const currDate = new Date();
    const dateDiffMs = expireDate.getTime() - currDate.getTime();
    return dateDiffMs < bufferInSeconds * 1000;
}
export function getBaseUrl() {
    return process.env['NANGO_SERVER_URL'] || localhostUrl;
}
export function getBasePublicUrl() {
    if (process.env['NANGO_SERVER_URL']) {
        return process.env['NANGO_SERVER_URL'].replace('api.', 'app.');
    }
    else {
        return getBaseUrl();
    }
}
/**
 * Get Oauth callback url base url.
 * @desc for ease of use with APIs that require a secure redirect
 * redirectmeto is automatically used. This is intentioned
 * for local development
 * @see https://github.com/kodie/redirectmeto
 */
export function getLocalOAuthCallbackUrlBaseUrl() {
    return 'https://redirectmeto.com/' + localhostUrl;
}
export function getApiUrl() {
    if (isStaging()) {
        return stagingHost;
    }
    else if (isEnterprise()) {
        return process.env['NANGO_SERVER_URL'];
    }
    else if (isProd()) {
        return cloudHost;
    }
    return getServerBaseUrl();
}
export function getGlobalOAuthCallbackUrl() {
    const baseUrl = process.env['NANGO_SERVER_URL'] || getLocalOAuthCallbackUrlBaseUrl();
    return baseUrl + '/oauth/callback';
}
export function getGlobalAppCallbackUrl() {
    const baseUrl = process.env['NANGO_SERVER_URL'] || getLocalOAuthCallbackUrlBaseUrl();
    return baseUrl + '/app-auth/connect';
}
export function getGlobalWebhookReceiveUrl() {
    const baseUrl = process.env['NANGO_SERVER_URL'] || getLocalOAuthCallbackUrlBaseUrl();
    return baseUrl + '/webhook';
}
export async function getOauthCallbackUrl(environmentId) {
    const globalCallbackUrl = getGlobalOAuthCallbackUrl();
    if (environmentId != null) {
        const environment = await environmentService.getByAccountIdAndEnvironment(environmentId);
        return environment?.callback_url || globalCallbackUrl;
    }
    return globalCallbackUrl;
}
export async function getAppCallbackUrl(_environmentId) {
    const globalAppCallbackUrl = getGlobalAppCallbackUrl();
    // TODO add this to settings and make it configurable
    return globalAppCallbackUrl;
}
/**
 * Get any custom path for the websockets server.
 * Defaults to '/' for backwards compatibility
 *
 * @returns the path for the websockets server
 */
export function getWebsocketsPath() {
    return process.env['NANGO_SERVER_WEBSOCKETS_PATH'] || '/';
}
/**
 * A helper function to interpolate a string.
 * interpolateString('Hello ${name} of ${age} years", {name: 'Tester', age: 234}) -> returns 'Hello Tester of age 234 years'
 *
 * @remarks
 * Copied from https://stackoverflow.com/a/1408373/250880
 */
export function interpolateString(str, replacers) {
    return str.replace(/\${([^{}]*)}/g, (a, b) => {
        const r = replacers[b];
        return typeof r === 'string' || typeof r === 'number' ? r : a; // Typecast needed to make TypeScript happy
    });
}
export function interpolateStringFromObject(str, replacers) {
    return str.replace(/\${([^{}]*)}/g, (a, b) => {
        const r = b.split('.').reduce((o, i) => o[i], replacers);
        return typeof r === 'string' || typeof r === 'number' ? r : a;
    });
}
export function connectionCopyWithParsedConnectionConfig(connection) {
    const connectionCopy = Object.assign({}, connection);
    const rawConfig = connectionCopy.connection_config;
    if (!rawConfig || Object.keys(rawConfig).length === 0) {
        return connectionCopy;
    }
    const parsedConfig = {};
    Object.keys(rawConfig).forEach(function (key) {
        const newKey = key.replace('connectionConfig.', '');
        const value = rawConfig[key];
        if (newKey && value) {
            parsedConfig[newKey] = value;
        }
    });
    connectionCopy.connection_config = parsedConfig;
    return connectionCopy;
}
export function mapProxyBaseUrlInterpolationFormat(baseUrl) {
    // Maps the format that is used in providers.yaml (inherited from oauth), to the format of the Connection model.
    return baseUrl ? baseUrl.replace('connectionConfig', 'connection_config') : baseUrl;
}
export function interpolateIfNeeded(str, replacers) {
    if (str.includes('${')) {
        return interpolateStringFromObject(str, replacers);
    }
    else {
        return str;
    }
}
export function setAccount(accountId, res) {
    res.locals[accountIdLocalsKey] = accountId;
}
export function setEnvironmentId(environmentId, res) {
    res.locals[environmentIdLocalsKey] = environmentId;
}
export function getAccount(res) {
    if (res.locals == null || !(accountIdLocalsKey in res.locals)) {
        throw new NangoError('account_not_set_in_locals');
    }
    const accountId = res.locals[accountIdLocalsKey];
    if (Number.isInteger(accountId)) {
        return accountId;
    }
    else {
        throw new NangoError('account_malformed_in_locals');
    }
}
export function getEnvironmentId(res) {
    if (res.locals === null || !(environmentIdLocalsKey in res.locals)) {
        throw new NangoError('environment_not_set_in_locals');
    }
    const environmentId = res.locals[environmentIdLocalsKey];
    if (Number.isInteger(environmentId)) {
        return environmentId;
    }
    else {
        throw new NangoError('environment_malformed_in_locals');
    }
}
export async function getEnvironmentAndAccountId(res, req) {
    if (req.user) {
        const { response: accountInfo, success, error } = await getAccountIdAndEnvironmentIdFromSession(req);
        if (!success || accountInfo == null) {
            return { response: null, error, success: false };
        }
        const response = { ...accountInfo, isWeb: true };
        return { response, error: null, success: true };
    }
    else {
        const accountId = getAccount(res);
        const environmentId = getEnvironmentId(res);
        const response = { accountId, environmentId, isWeb: false };
        return Promise.resolve({ response, error: null, success: true });
    }
}
export async function getAccountIdAndEnvironmentIdFromSession(req) {
    const sessionUser = req.user;
    const currentEnvironment = req.cookies['env'] || 'dev';
    if (sessionUser == null) {
        const error = new NangoError('user_not_found');
        return { response: null, error, success: false };
    }
    const user = await userService.getUserById(sessionUser.id);
    if (user == null) {
        const error = new NangoError('user_not_found');
        return { response: null, error, success: false };
    }
    const environmentAndAccount = await environmentService.getAccountAndEnvironmentById(user.account_id, currentEnvironment);
    if (environmentAndAccount == null) {
        const error = new NangoError('account_not_found');
        return { response: null, error, success: false };
    }
    const { account, environment } = environmentAndAccount;
    const response = { accountId: account.id, environmentId: environment.id };
    return { response, error: null, success: true };
}
export function isApiAuthenticated(res) {
    return res.locals != null && accountIdLocalsKey in res.locals && Number.isInteger(res.locals[accountIdLocalsKey]);
}
export function isUserAuthenticated(req) {
    const user = req.user;
    return typeof req.isAuthenticated === 'function' && req.isAuthenticated() && user != null && user.id != null;
}
export function getConnectionConfig(queryParams) {
    const arr = Object.entries(queryParams).filter(([, v]) => typeof v === 'string'); // Filter strings
    return Object.fromEntries(arr);
}
export function packageJsonFile() {
    const localPath = process.env['SERVER_RUN_MODE'] === 'DOCKERIZED' ? 'packages/shared/package.json' : '../shared/package.json';
    return JSON.parse(readFileSync(resolve(process.cwd(), localPath)).toString('utf-8'));
}
export function safeStringify(obj) {
    const stringify = (obj, indent = 2) => {
        const cache = new Set();
        const jsonString = JSON.stringify(obj, (_key, value) => {
            if (typeof value === 'object' && value !== null) {
                if (cache.has(value)) {
                    return;
                }
                cache.add(value);
            }
            return value;
        }, indent);
        cache.clear();
        return jsonString;
    };
    const content = obj.map((arg) => (typeof arg === 'object' ? stringify(arg) : String(arg))).join(' ');
    return content;
}
//# sourceMappingURL=utils.js.map